#if !defined  HAVE_WALSHHARTLEY_H__
#define       HAVE_WALSHHARTLEY_H__


#include "revbinpermute.h"
#include "reverse.h" // reverse()


template <typename Type>
void walsh_hartley_05_dif_core(Type *f, ulong ldn)
// decimation in frequency (DIF) algorithm
// =^=
//  gray_permute(f, n);  walsh_wak(f,ldn);
{
    const ulong n = (1<<ldn);

    for (ulong ldm=ldn; ldm>0; --ldm) // dif
    {
        const ulong m = (1<<ldm);
        const ulong mh = (m>>1);
        for (ulong r=0; r<n; r+=m)
        {
            ulong t1 = r;
            ulong t2 = r+mh;
            Type *f2 = f+t2;
            t2 = r + m - 1;
//            for (ulong j=0; j<mh; ++j,++t1,--t2)
            for (  ;  t1<t2;  ++t1,--t2)
            {
                Type u = f[t1];
                Type v = f[t2];
                f[t1] = u + v;
                f[t2] = u - v;
            }
            reverse(f2, mh);
        }
    }
}
// -------------------------


template <typename Type>
void walsh_hartley_05_dit_core(Type *f, ulong ldn)
// decimation in time (DIT) algorithm
// =^=
//  walsh_wak(f,ldn);  inverse_gray_permute(f, n);
{
    const ulong n = (1<<ldn);

    for (ulong ldm=1; ldm<=ldn; ++ldm) // dit
    {
        const ulong m = (1<<ldm);
        const ulong mh = (m>>1);
        for (ulong r=0; r<n; r+=m)
        {
            ulong t1 = r;
            ulong t2 = r+mh;
            Type *f2 = f+t2;
            reverse(f2, mh);
            t2 = r + m - 1;
//            for (ulong j=0; j<mh; ++j,++t1,--t2)
            for (  ;  t1<t2;  ++t1,--t2)
            {
                Type u = f[t1];
                Type v = f[t2];
                f[t1] = u + v;
                f[t2] = u - v;
            }
        }
    }
}
// -------------------------


template <typename Type>
void walsh_hartley_05(Type *f, ulong ldn)
// transform wrt. the basis (sequency at end of lines), '*':=1, ' ':=-1:
// 0: [* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *] ( 0)
// 1: [* * * * * * * *                                 * * * * * * * *] ( 2)
// 2: [* * * *                 * * * * * * * *                 * * * *] ( 4)
// 3: [* * * *         * * * *                 * * * *         * * * *] ( 6)
// 4: [* *         * * * *         * * * *         * * * *         * *] ( 8)
// 5: [* *         * *     * * * *         * * * *     * *         * *] (10)
// 6: [* *     * *         * *     * * * *     * *         * *     * *] (12)
// 7: [* *     * *     * *     * *         * *     * *     * *     * *] (14)
// 8: [*     * *     * *     * *     * *     * *     * *     * *     *] (16)
// 9: [*     * *     *   * *     * *     * *     * *   *     * *     *] (18)
//10: [*     *   * *     * *   *     * *     *   * *     * *   *     *] (20)
//11: [*     *   * *   *     *   * *     * *   *     *   * *   *     *] (22)
//12: [*   *     *   * *   *     *   * *   *     *   * *   *     *   *] (24)
//13: [*   *     *   *   *   * *   *     *   * *   *   *   *     *   *] (26)
//14: [*   *   *   *     *   *   *   * *   *   *   *     *   *   *   *] (28)
//15: [*   *   *   *   *   *   *   *     *   *   *   *   *   *   *   *] (30)
//16: [*   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *  ] (31)
//17: [*   *   *   *     *   *   *   *   *   *   *   * *   *   *   *  ] (29)
//18: [*   *     *   *   *   * *   *   *   *     *   *   *   * *   *  ] (27)
//19: [*   *     *   * *   *     *   *   *   * *   *     *   * *   *  ] (25)
//20: [*     *   * *   *     *   * *   *     *   * *   *     *   * *  ] (23)
//21: [*     *   * *     * *   *     *   * *   *     * *     *   * *  ] (21)
//22: [*     * *     *   * *     * *   *     * *     *   * *     * *  ] (19)
//23: [*     * *     * *     * *     *   * *     * *     * *     * *  ] (17)
//24: [* *     * *     * *     * *     * *     * *     * *     * *    ] (15)
//25: [* *     * *         * *     * *     * *     * * * *     * *    ] (13)
//26: [* *         * *     * * * *     * *         * *     * * * *    ] (11)
//27: [* *         * * * *         * *     * * * *         * * * *    ] ( 9)
//28: [* * * *         * * * *         * * * *         * * * *        ] ( 7)
//29: [* * * *                 * * * *         * * * * * * * *        ] ( 5)
//30: [* * * * * * * *                 * * * * * * * *                ] ( 3)
//31: [* * * * * * * * * * * * * * * *                                ] ( 1)
//
// self-inverse
//
// perfectly equivalent to shifted by 0.5 hartley transform
{
#if  1
    revbin_permute(f, 1<<ldn);
    walsh_hartley_05_dif_core(f, ldn);
#else
    walsh_hartley_05_dit_core(f, ldn);
    revbin_permute(f, 1<<ldn);
#endif
    // =^=
//    zip_rev(f, n);
//    walsh_wal(f, ldn);
    // =^=
//    walsh_wal(f, ldn);
//    unzip_rev(f, n);
    // =^=
//    walsh_wak(f, ldn);
//    inverse_gray_permute(f, n);
//    revbin_permute(f, n);
}
// -------------------------

// yet another nice walsh transform:
// for (ulong k=0; k<n; ++k)  sumdiff05(f[k], f[n-1-k]);
// walsh_hartley_05(f, ldn);



// basis for 0.5-shifted hartley transform:
//  0: [* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *] ( 0)
//  1: [* * * * * * * * * * * *                                 * * * *] ( 2)
//  2: [* * * * * *                 * * * * * * * *                 * *] ( 4)
//  3: [* * * *           * * * * * *           * * * * *             *] ( 6)
//  4: [* * *         * * * *         * * * *         * * * *         *] ( 8)
//  5: [* *         * * *       * * *       * * * *       * * *       *] (10)
//  6: [* *       * *       * * *     * * *       * *       * * *     *] (12)
//  7: [* *     * *       * *     * *       * *     * * *     * *     *] (14)
//  8: [*       *       *       *       *       *       *       *      ] (15)
//  9: [*     * *     *     * *     * *   * *     * *   * *     * *    ] (17)
// 10: [*     *     * *   * *   * *     *     *     * *   * *   * *    ] (19)
// 11: [*     *   * *   * *   * *   * *   * *   *     *     *     *    ] (21)
// 12: [*   * *   *     *   * *   *     *   * *   *     *   * *   *    ] (23)
// 13: [*   *     *   *     *   *   * *   *   * *   *   * *   *   *    ] (25)
// 14: [*   *   *     *   *   *   * *   *   *   *     *   *   *   * *  ] (27)
// 15: [*   *   *   *   *   *     *   *   *   *   *   *   *   * *   *  ] (29)
// 16: [*   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *  ] (31)
// 17: [*   *     *   *   *   *   *   *   *   * *   *   *   *   *   *  ] (29)
// 18: [*     *   *   *   * *   *   *   *     *   *   *   * *   *   *  ] (27)
// 19: [* *   *   *     *   *     *   *     *   *   * *   *   * *   *  ] (25)
// 20: [* *   *     *   * *   *     *   * *   *     *   * *   *     *  ] (23)
// 21: [* *   * *   * *   * *   *     *     *     *     *     *   * *  ] (21)
// 22: [* *     *     *     * *   * *   * *     *     *     * *   * *  ] (19)
// 23: [* *     * *     *     * *     *     * *     * *   * *     * *  ] (17)
// 24: [  *       *       *       *       *       *       *       *    ] (17)
// 25: [  * *     * *       * *     * * *     * *     * * *     * *    ] (14)
// 26: [  * *       * * *     * * *       * *       * * *     * * *    ] (12)
// 27: [  * * *       * * *         * * *       * * *       * * * *    ] (10)
// 28: [  * * * *         * * * *         * * * *         * * * *      ] ( 8)
// 29: [  * * * * * *           * * * * *             * * * * *        ] ( 6)
// 30: [    * * * * * * * *                 * * * * * * * *            ] ( 4)
// 31: [        * * * * * * * * * * * * * * * *                        ] ( 2)


// base for hartley transform:
//  0: [* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *] ( 0)
//  1: [* * * * * * * * * * * *                                   * * *] ( 2)
//  2: [* * * * * *                   * * * * * * *                   *] ( 4)
//  3: [* * * *             * * * * *             * * * * *           *] ( 6)
//  4: [* * *           * * *           * * *           * * *          ] ( 7)
//  5: [* * *       * * *         * * *       * * *       * * *        ] ( 9)
//  6: [* *       * * *       * *       * *       * * *       * *      ] (11)
//  7: [* *       * *     * *       * *     * *       * *     * * *    ] (13)
//  8: [* *     * *     * *     * *     * *     * *     * *     * *    ] (15)
//  9: [* *     *     * *     *     * *     * *   * *     * *     *    ] (17)
// 10: [* *   * *     *     *     *     * *   * *     *     *     *    ] (19)
// 11: [* *   *     *     *     *     *     *     *   * *   * *   * *  ] (21)
// 12: [*     *     *   *     *     *   *     *     *   *     *     *  ] (23)
// 13: [*     *   *     *   *     *   *   * *   *   * *   *   *     *  ] (25)
// 14: [*     *   *   *   *     *   *   *     *   *   *   *     *   *  ] (27)
// 15: [*   *     *   *   *   *   *   *   *   *     *   *   *   *   *  ] (29)
// 16: [*   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *  ] (31)
// 17: [*   *   *   *   *   *     *   *   *   *   *   *   *   *     *  ] (29)
// 18: [*   *   *     *   *   *   *     *   *   *     *   *   *   *    ] (27)
// 19: [*   *     *   *   * *   *   * *   *   *     *   *     *   *    ] (25)
// 20: [*   *     *     *   *     *     *   *     *     *   *     *    ] (23)
// 21: [*   * *   * *   * *   *     *     *     *     *     *     *   *] (22)
// 22: [*     *     *     *     * *   * *     *     *     *     * *   *] (20)
// 23: [*     *     * *     * *   * *     * *     *     * *     *     *] (18)
// 24: [*     * *     * *     * *     * *     * *     * *     * *     *] (16)
// 25: [*     * * *     * *       * *     * *       * *     * *       *] (14)
// 26: [*       * *       * * *       * *       * *       * * *       *] (12)
// 27: [*         * * *       * * *       * * *         * * *       * *] (10)
// 28: [*           * * *           * * *           * * *           * *] ( 8)
// 29: [* *           * * * * *             * * * * *             * * *] ( 6)
// 30: [* *                   * * * * * * *                   * * * * *] ( 4)
// 31: [* * * *                                   * * * * * * * * * * *] ( 2)


#endif  // !defined HAVE_WALSHHARTLEY_H__
